<?php
require 'vendor/autoload.php';
require_once('vendor/PHPMailer/phpmailer.class.php');

use AlibabaCloud\Client\AlibabaCloud;
use AlibabaCloud\Client\Exception\ClientException;
use AlibabaCloud\Client\Exception\ServerException;

class AliyunTrafficCheck
{
    private $accounts;
    private $notificationEmail;
    private $notificationTitle;
    private $notificationToken;

    public function __construct()
    {
        $config = include 'config.php';
        $this->accounts = $config['Accounts'];
        $this->notificationEmail = $config['Notification']['email'];
        $this->notificationTitle = $config['Notification']['title'];
        $this->notificationHost = $config['Notification']['host'];
        $this->notificationUsername = $config['Notification']['username'];
        $this->notificationPassword = $config['Notification']['password'];
        $this->notificationPort = $config['Notification']['port'];
        $this->notificationSecure = $config['Notification']['secure'];
    }

    public function getTraffic($accessKeyId, $accessKeySecret)
    {
        try {
            AlibabaCloud::accessKeyClient($accessKeyId, $accessKeySecret)
                ->regionId('cn-hongkong')
                ->asDefaultClient();

            $result = AlibabaCloud::rpc()
                ->product('CDT')
                ->version('2021-08-13')
                ->action('ListCdtInternetTraffic')
                ->method('POST')
                ->host('cdt.aliyuncs.com')
                ->request();

            $total = array_sum(array_column($result['TrafficDetails'], 'Traffic'));
            return $total / (1024 * 1024 * 1024); // Convert to GB
        } catch (ClientException $e) {
            echo '客户端异常: ' . $e->getErrorMessage() . PHP_EOL;
        } catch (ServerException $e) {
            echo '服务器异常: ' . $e->getErrorMessage() . PHP_EOL;
        }
        return 0;
    }

    public function check()
    {
        $logs = [];
        foreach ($this->accounts as $account) {
            $traffic = $this->getTraffic($account['AccessKeyId'], $account['AccessKeySecret']);
            $maskedAccessKeyId = substr($account['AccessKeyId'], 0, 7) . '***';
            $usagePercentage = round(($traffic / $account['maxTraffic']) * 100, 2);
            $regionName = $this->getRegionName($account['regionId']);
            $log = [
                '账号' => $maskedAccessKeyId,
                '总流量' => $account['maxTraffic'] . 'GB',
                '已使用流量' => $traffic . 'GB',
                '使用百分比' => $usagePercentage . '%',
                '地区' => $regionName,
                '使用率达到95%' => $usagePercentage >= 95 ? '是' : '否'
            ];
            
            if ($usagePercentage >= 95) {
                $this->controlInstance($account, 'stop');
                $notificationResult = $this->sendNotification($account['AccessKeyId'], $traffic);
                $log['通知发送'] = $notificationResult === true ? '成功' : "失败: {$notificationResult}";
            } else {
                $log['通知发送'] = '不需要';
            }
            $logs[] = $log;
        }
        $this->writeLog($logs);
    }

    private function writeLog($logs)
    {
        $data = [
            '获取时间' => date('Y-m-d H:i:s'),
            '日志' => $logs
        ];

        // 将数据写入 data.json 文件
        $jsonData = json_encode($data, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT);
        file_put_contents('data.json', $jsonData);

        // 输出数据到浏览器
        header('Content-Type: application/json');
        echo $jsonData;
    }

    private function sendNotification($accessKeyId, $traffic)
    {
        $config = include 'config.php';
        $emailConfig = $config['Notification'];

        $message = "账号: {$accessKeyId}<br>流量: {$traffic}GB<br>已达到 95% 使用率";

        return $this->send_mail(
            $this->notificationEmail,
            '',
            $this->notificationTitle,
            $message,
            null,
            $emailConfig
        );
    }

    private function send_mail($to, $name, $subject = '', $body = '', $attachment = null, $config = '')
    {
        $config = is_array($config) ? $config : array();
        $mail = new PHPMailer();
        $mail->CharSet = 'UTF-8';
        $mail->IsSMTP();
        $mail->SMTPDebug = 0;
        $mail->SMTPAuth = true;
        $mail->SMTPSecure = $this->notificationSecure;
        $mail->Host = $this->notificationHost;
        $mail->Port = $this->notificationPort;
        $mail->Username = $this->notificationUsername;
        $mail->Password = $this->notificationPassword;
        $mail->SetFrom($this->notificationUsername, '阿里云CDT告警');
        $mail->Subject = $subject;
        $mail->MsgHTML($body);
        $mail->AddAddress($to, $name);
        return $mail->Send() ? true : $mail->ErrorInfo;
    }

    public function performAction($account, $action)
    {
        try {
            AlibabaCloud::accessKeyClient($account['AccessKeyId'], $account['AccessKeySecret'])
                ->regionId($account['regionId'])
                ->asDefaultClient();

            $result = AlibabaCloud::rpc()
                ->product('Ecs')
                ->version('2014-05-26')
                ->action('DescribeInstanceStatus')
                ->method('POST')
                ->host("ecs.{$account['regionId']}.aliyuncs.com")
                ->options([
                    'query' => [
                        'RegionId' => $account['regionId'],
                        'InstanceId' => $account['instanceId']
                    ]
                ])
                ->request();

            $instanceStatus = $result['InstanceStatuses']['InstanceStatus'][0]['Status'];

            if ($action === 'stop' && $instanceStatus !== 'Stopped') {
                AlibabaCloud::rpc()
                    ->product('Ecs')
                    ->version('2014-05-26')
                    ->action('StopInstance')
                    ->method('POST')
                    ->host("ecs.{$account['regionId']}.aliyuncs.com")
                    ->options([
                        'query' => [
                            'RegionId' => $account['regionId'],
                            'InstanceId' => $account['instanceId']
                        ]
                    ])
                    ->request();
            } elseif ($action === 'start' && $instanceStatus !== 'Running') {
                AlibabaCloud::rpc()
                    ->product('Ecs')
                    ->version('2014-05-26')
                    ->action('StartInstance')
                    ->method('POST')
                    ->host("ecs.{$account['regionId']}.aliyuncs.com")
                    ->options([
                        'query' => [
                            'RegionId' => $account['regionId'],
                            'InstanceId' => $account['instanceId']
                        ]
                    ])
                    ->request();
            }
        } catch (ClientException $e) {
            echo '客户端异常: ' . $e->getErrorMessage() . PHP_EOL;
        } catch (ServerException $e) {
            echo '服务器异常: ' . $e->getErrorMessage() . PHP_EOL;
        }
    }

    public function controlInstance($account, $action)
    {
        try {
            AlibabaCloud::accessKeyClient($account['AccessKeyId'], $account['AccessKeySecret'])
                ->regionId($account['regionId'])
                ->asDefaultClient();

            if (!empty($account['instanceId'])) {
                $this->performAction($account, $action);
            } else {
                $result = AlibabaCloud::rpc()
                    ->product('Ecs')
                    ->version('2014-05-26')
                    ->action('DescribeInstances')
                    ->method('POST')
                    ->host("ecs.{$account['regionId']}.aliyuncs.com")
                    ->options([
                        'query' => [
                            'RegionId' => $account['regionId']
                        ]
                    ])
                    ->request();

                foreach ($result['Instances']['Instance'] as $instance) {
                    $account['instanceId'] = $instance['InstanceId'];
                    $this->performAction($account, $action);
                }
            }
        } catch (ClientException $e) {
            echo '客户端异常: ' . $e->getErrorMessage() . PHP_EOL;
        } catch (ServerException $e) {
            echo '服务器异常: ' . $e->getErrorMessage() . PHP_EOL;
        }
    }

    private function getRegionName($regionId)
    {
        $regions = [
            'cn-qingdao' => '华北1(青岛)',
            'cn-beijing' => '华北2(北京)',
            'cn-zhangjiakou' => '华北3(张家口)',
            'cn-huhehaote' => '华北5(呼和浩特)',
            'cn-wulanchabu' => '华北6(乌兰察布)',
            'cn-hangzhou' => '华东1(杭州)',
            'cn-shanghai' => '华东2(上海)',
            'cn-nanjing' => '华东5 (南京-本地地域)',
            'cn-fuzhou' => '华东6(福州-本地地域)',
            'cn-wuhan-lr' => '华中1(武汉-本地地域)',
            'cn-shenzhen' => '华南1(深圳)',
            'cn-heyuan' => '华南2(河源)',
            'cn-guangzhou' => '华南3(广州)',
            'cn-chengdu' => '西南1(成都)',
            'cn-hongkong' => '中国香港',
            'ap-southeast-1' => '新加坡',
            'ap-southeast-2' => '澳大利亚(悉尼)',
            'ap-southeast-3' => '马来西亚(吉隆坡)',
            'ap-southeast-5' => '印度尼西亚(雅加达)',
            'ap-southeast-6' => '菲律宾(马尼拉)',
            'ap-southeast-7' => '泰国(曼谷)',
            'ap-northeast-1' => '日本(东京)',
            'ap-northeast-2' => '韩国(首尔)',
            'us-west-1' => '美国(硅谷)',
            'us-east-1' => '美国(弗吉尼亚)',
            'eu-central-1' => '德国(法兰克福)',
            'eu-west-1' => '英国(伦敦)',
            'me-east-1' => '阿联酋(迪拜)',
            'me-central-1' => '沙特(利雅得)'
        ];

        return isset($regions[$regionId]) ? $regions[$regionId] : '未知地区';
    }
}

$aliyunTrafficCheck = new AliyunTrafficCheck();
$aliyunTrafficCheck->check();